from azdocli.lib.commons import create_azdo_connection
from azdocli.lib.exceptions import exception_handler


class CoreAPI:
    @exception_handler
    def __init__(self, org_name, org_pat):
        self.connection = create_azdo_connection(org_name, org_pat)
        self.core_client = self.connection.clients_v6_0.get_core_client()

    @exception_handler
    def list_projects(self):
        """

        :return:
        """
        list_of_projects = {
            'count': 0,
            'values': []
        }

        for p in self.core_client.get_projects():
            list_of_projects['count'] += 1
            list_of_projects['values'].append(
                {
                    'id': p.id,
                    'name': p.name
                }
            )

        return list_of_projects

    @exception_handler
    def get_project(self, project_name):
        """

        :param project_name:
        :return:
        """

        project = self.core_client.get_project(project_name)
        project_result = {
            "id": project.id,
            "name": project.name,
            "description": project.description if project.description else ""
        }

        return project_result

    @exception_handler
    def list_teams(self):
        """
        List all teams in the organization
        :return:
        """
        list_of_all_teams = {
            'count': 0,
            'values': []
        }

        all_teams = self.core_client.get_all_teams()
        for team in all_teams:
            list_of_all_teams['count'] += 1
            list_of_all_teams['values'].append(
                {
                    "project_name": team.project_name,
                    "team_name": team.name
                }
            )

        return list_of_all_teams

    @exception_handler
    def get_teams(self, project_name):
        """
        Get teams of a project
        :param project_name:
        :return:
        """

        result = {
            'count': 0,
            'values': []
        }

        teams_of_project = self.core_client.get_teams(project_name)
        for t in teams_of_project:
            result['count'] += 1
            result['values'].append(
                {
                    "team_name": t.name,
                    "team_id": t.id
                }
            )

        return result

    @exception_handler
    def get_team(self, project_name, team_id):
        """
        Get a specific team
        :param project_name:
        :param team_id:
        :return:
        """

        team = self.core_client.get_team(project_name, team_id)
        team_result = {
            "id": team.id,
            "name": team.name,
        }

        return team_result



from abc import ABC, abstractmethod


class Context():

    def __init__(self, strategy=None):
        self._strategy = strategy

    @property
    def strategy(self):
        return self._strategy

    @strategy.setter
    def strategy(self, strategy):
        self._strategy = strategy

    def execute(self):
        result = self._strategy.run_strategy()
        return result


class Strategy(ABC):

    @abstractmethod
    def run_strategy(self):
        pass


class StrategyA(Strategy):
    def run_strategy(self):
        return 1


class StrategyB(Strategy):
    def run_strategy(self):
        return 2


if __name__ == "__main__":
    context = Context()
    context.strategy = StrategyA()
    print(context.execute())
    context.strategy = StrategyB()
    print(context.execute())


from azdocli.strategies.strategy_context import Strategy
from azdocli.lib.core import CoreAPI


class TeamsGetStrategy(Strategy):
    def run_strategy(self):
        coreapi = CoreAPI(self.org_name, self.org_pat)
        if self.ctx['team_id'] == '':
            return coreapi.get_teams(self.ctx['project_name'])
        else:
            return coreapi.get_team(self.ctx['project_name'], self.ctx['team_id'])


from azdocli.strategies.strategy_context import StrategyContext
from azdocli.strategies.projects_list_strategy import ProjectsListStrategy
from azdocli.strategies.projects_get_strategy import ProjectsGetStrategy
from azdocli.strategies.svc_list_strategy import SvcListStrategy
from azdocli.strategies.teams_list_strategy import TeamsListStrategy
from azdocli.strategies.teams_get_strategy import TeamsGetStrategy
from azdocli.strategies.pipelineruns_get_strategy import PipelinerunsGetStrategy


def execute_strategy(ctx):
    controller = ctx.obj['controller']
    operation = ctx.obj['operation']
    context = StrategyContext()

    if controller == 'projects' and operation == 'getall':
        context.strategy = ProjectsListStrategy(ctx)
        return context.execute()
    elif controller == 'projects' and operation == 'get':
        context.strategy = ProjectsGetStrategy(ctx)
        return context.execute()
    elif controller == 'svc' and operation == 'getall':
        context.strategy = SvcListStrategy(ctx)
        return context.execute()
    elif controller == 'teams' and operation == 'getall':
        context.strategy = TeamsListStrategy(ctx)
        return context.execute()
    elif controller == 'teams' and operation == 'get':
        context.strategy = TeamsGetStrategy(ctx)
        return context.execute()
    elif controller == 'pipelineruns' and operation == 'get':
        context.strategy = PipelinerunsGetStrategy(ctx)
        return context.execute()
    else:
        print("No strategies found")
        return None


from azdocli.strategies.strategy_context import Strategy
from azdocli.lib.core import CoreAPI


class ProjectsGetStrategy(Strategy):
    def run_strategy(self):
        coreapi = CoreAPI(self.org_name, self.org_pat)
        return coreapi.get_project(self.ctx['project_name'])


from azdocli.strategies.strategy_context import Strategy
from azdocli.lib.pipelines import PipelinesAPI


class PipelinerunsGetStrategy(Strategy):
    def run_strategy(self):
        pipelinesapi = PipelinesAPI(self.org_name, self.org_pat)
        return pipelinesapi.get_pipelineruns(self.ctx['project_name'], self.ctx['pipeline_id'])


import unittest

from azdocli.lib import commons


class TestCommons(unittest.TestCase):

    def test_if_settings_file_does_not_exist_then_it_must_return_none(self):
        expected = None
        actual = commons.load_settings('./tests/lib/dummy/settings.ini')
        self.assertEqual(expected, actual)

    def test_if_settings_file_exists_then_it_must_return_correct_config_values(self):
        loaded = commons.load_settings('./tests/lib/test-settings.ini')

        expected_org_name = 'dummy_org_name'
        expected_pat = 'dummy_pat'
        actual_org_name = loaded['org']['name']
        actual_pat = loaded['org']['pat']

        self.assertEqual(expected_org_name, actual_org_name)
        self.assertEqual(expected_pat, actual_pat)

    def test_if_settings_file_exists_but_does_not_contain_correct_values_it_must_return_none(self):
        expected = None
        actual = commons.load_settings('./tests/lib/settings2.ini')
        self.assertEqual(expected, actual)


if __name__ == '__main__':
    unittest.main()


import configparser

from azure.devops.connection import Connection

from msrest.authentication import BasicAuthentication


def foo_commons():
    return "Hello commons"


def create_azdo_connection(org_name, org_pat):
    """
    Creates the connection to the Azure DevOps orgnanization that will be used by the clients

    :param str org_name: Name of the organization
    :param str org_pat:  Personal access token for the organization
    :return: azure.devops.connection.Connection
    """

    organization_url = f'https://dev.azure.com/{org_name}'
    credentials = BasicAuthentication('', org_pat)

    connection = Connection(base_url=organization_url, creds=credentials)

    return connection


def load_settings(filename='settings.ini'):
    """
    Loads the settings.ini file which contains the organization name and personal access token

    :param str filename: location of the file
    :return: config object or None
    """

    config = configparser.ConfigParser()
    config.read(filename)

    if not config.sections():
        return None
    else:
        try:
            if config['org']['name'] is not None and config['org']['pat'] is not None:
                return config
        except KeyError:
            return None



import configparser
from pprint import pprint

import click

from azdocli.lib.commons import load_settings
from azdocli.lib.controller_switch import execute_strategy


def set_context(ctx, operation, filename='settings.ini'):
    """
    Always done before performing an operation (get, getall, etc) to ensure that the organization name and pat is
      initialized in the context.

    :param ctx: context object passed around
    :param str operation: operation to be performed (get, getall, etc)
    :param str filename: location of the settings.ini file
    :return: ctx or None
    """
    ctx.obj['operation'] = operation
    cfg = load_settings(filename)
    if not cfg:
        return None
    else:
        ctx.obj['org_name'] = cfg['org']['name']
        ctx.obj['org_pat'] = cfg['org']['pat']
        return ctx


def run(ctx, operation, filename='settings.ini'):
    """
    Sets the context then executes the strategy

    :param ctx: context object passed around
    :param str operation: operation to be performed (get, getall, etc)
    :param str filename: location of the settings.ini file
    :return:
    """
    if set_context(ctx, operation, filename):
        result = execute_strategy(ctx)
        if result:
            pprint(result)
    else:
        click.echo("The settings.ini file is not found or is not initialized properly. Please use 'azdocli init' or "
                   "refer to the documentation.")


@click.group()
def cli():
    """
    azdocli is a command line interface for Azure DevOps. The user can perform GET operations for different
    resources directly in their terminal. The same can be achieved by using Postman or the web browser, but
    where is the fun in that.

    It uses a Personal Access Token (PAT) to connect to the API which the user must specify by running:

        azdocli init

    Once the settings.ini file is created, commands now can be ran such as:

        azdocli projects getall

    This will display all the projects that the user has access to in a given organization.

    For any issues or help needed: https://github.com/migzpogi
    """
    pass


@click.command()
@click.option('--orgname', prompt='The name of the Azure DevOps organization',
              help='The name of the Azure Devops organization')
@click.option('--pat', prompt='Your personal access token for that organization',
              help='Your personal access token for that organization')
@click.option('--filename', default='settings.ini',
              help='Location and filename.')
def init(orgname, pat, filename):
    """
    Initialize the settings.ini file
    """
    config = configparser.ConfigParser()
    config['org'] = {
        'name': orgname,
        'pat': pat
    }
    with open(filename, 'w') as f:
        config.write(f)


@click.group()
@click.pass_context
def projects(ctx):
    """
    Commands for managing projects
    """
    ctx.ensure_object(dict)
    ctx.obj['controller'] = 'projects'
    pass


@click.group()
@click.pass_context
def svc(ctx):
    """
    Commands for managing service endpoints
    """
    ctx.ensure_object(dict)
    ctx.obj['controller'] = 'svc'


@click.group()
@click.pass_context
def teams(ctx):
    """
    Commands for managing teams
    """
    ctx.ensure_object(dict)
    ctx.obj['controller'] = 'teams'


@click.group()
@click.pass_context
def pipelineruns(ctx):
    """
    Commands for managing pipeline runs
    """
    ctx.ensure_object(dict)
    ctx.obj['controller'] = 'pipelineruns'


@click.command()
@click.pass_context
@click.option('--filename', default='settings.ini',
              help='Location and filename.')
def getall(ctx, filename):
    """
    Performs a list operation
    """
    run(ctx, 'getall', filename)


@click.command()
@click.pass_context
@click.option('--projectname', prompt='The name of the project.', help='The name of the project.')
@click.option('--pipelineid', default='', help="ID for pipeline commands.")
@click.option('--teamid', default='', help="ID for team commands.")
def get(ctx, projectname, pipelineid, teamid):
    """
    Performs a get operation
    """
    ctx.obj['project_name'] = projectname
    ctx.obj['pipeline_id'] = pipelineid
    ctx.obj['team_id'] = teamid
    run(ctx, 'get')


for command in [init, projects, svc, teams, pipelineruns]:
    cli.add_command(command)


projects.add_command(getall)
projects.add_command(get)

svc.add_command(getall)
svc.add_command(get)

teams.add_command(getall)
teams.add_command(get)

pipelineruns.add_command(get)


import unittest
import json

from click.testing import CliRunner
from azdocli.azdocli import cli

from azure.devops.exceptions import AzureDevOpsServiceError, AzureDevOpsAuthenticationError


class TestIntegrationProject(unittest.TestCase):
    def test_if_getall_projects_then_count_must_be_two(self):
        """001"""
        runner = CliRunner()
        result = runner.invoke(cli, ['projects', 'getall'])

        actual = json.loads(result.output.replace("'", '"'))

        self.assertEqual(2, actual['count'])
        self.assertEqual(result.exit_code, 0)

    def test_if_get_project_then_id_must_be_correct(self):
        """002"""
        runner = CliRunner()
        result = runner.invoke(cli, ['projects',
                                     'get',
                                     '--projectname',
                                     'leonhard'])

        actual = json.loads(result.output.replace("'", '"'))

        self.assertEqual('leonhard', actual['name'])
        self.assertEqual(result.exit_code, 0)

    def test_if_getall_projects_with_wrong_pat_then_exit_2(self):
        """003"""
        filename = './tests_integration/settings-wrong-creds.ini'
        runner = CliRunner()
        result = runner.invoke(cli, ['projects', 'getall', '--filename', filename])
        self.assertEqual(result.exit_code, 2)

    def test_if_getall_projects_with_invalid_pat_then_exit_1(self):
        """004"""
        filename = './tests_integration/settings-invalid-creds.ini'
        runner = CliRunner()
        result = runner.invoke(cli, ['projects', 'getall', '--filename', filename])
        self.assertEqual(result.exit_code, 1)

    def test_if_get_project_with_wrong_pat_then_exit_2(self):
        """005"""
        filename = './tests_integration/settings-wrong-creds.ini'
        runner = CliRunner()
        result = runner.invoke(cli, ['projects', 'get', '--projectname', 'leonhard', '--filename', filename])
        self.assertEqual(result.exit_code, 2)

    def test_if_get_project_with_invalid_pat_then_exit_2(self):
        """006"""
        filename = './tests_integration/settings-invalid-creds.ini'
        runner = CliRunner()
        result = runner.invoke(cli, ['projects', 'get', '--projectname', 'leonhard', '--filename', filename])
        self.assertEqual(result.exit_code, 2)

    def test_if_get_project_and_wrong_project_name_then_exit_1(self):
        """007"""
        runner = CliRunner()
        result = runner.invoke(cli, ['projects',
                                     'get',
                                     '--projectname',
                                     'noprojectname'])

        self.assertEqual(result.exit_code, 1)

    def test_if_getall_teams_then_count_must_be_three(self):
        """008"""
        runner = CliRunner()
        result = runner.invoke(cli, ['teams', 'getall'])

        actual = json.loads(result.output.replace("'", '"'))

        self.assertEqual(3, actual['count'])
        self.assertEqual(result.exit_code, 0)

    def test_if_get_teams_of_sampleproject_then_count_must_be_two(self):
        """008"""
        runner = CliRunner()
        result = runner.invoke(cli, ['teams', 'get', '--projectname', 'sampleproject'])

        actual = json.loads(result.output.replace("'", '"'))

        self.assertEqual(2, actual['count'])
        self.assertEqual(result.exit_code, 0)

    def test_if_get_teams_and_wrong_project_name_then_exit_1(self):
        """007"""
        runner = CliRunner()
        result = runner.invoke(cli, ['teams',
                                     'get',
                                     '--projectname',
                                     'noprojectname'])

        self.assertEqual(result.exit_code, 1)

    def test_if_get_specific_team_then_exit_code_must_be_zero(self):
        """011"""
        runner = CliRunner()
        result = runner.invoke(cli, ['teams', 'get', '--projectname', 'sampleproject', '--teamid', 'sampleproject Team'])

        self.assertEqual(result.exit_code, 0)

    def test_if_get_specific_team_that_does_not_exist_then_exit_code_must_be_zero(self):
        """012"""
        runner = CliRunner()
        result = runner.invoke(cli, ['teams', 'get', '--projectname', 'sampleproject', '--teamid', 'team does not exist'])

        self.assertEqual(result.exit_code, 1)

    def test_if_get_specific_team_and_project_does_not_exist_then_exit_code_must_be_zero(self):
        """013"""
        runner = CliRunner()
        result = runner.invoke(cli, ['teams', 'get', '--projectname', 'notexist', '--teamid', 'sampleproject Team'])

        self.assertEqual(result.exit_code, 1)


if __name__ == '__main__':
    unittest.main()


from abc import ABC, abstractmethod


class StrategyContext():

    def __init__(self, strategy=None):
        self._strategy = strategy

    @property
    def strategy(self):
        return self._strategy

    @strategy.setter
    def strategy(self, strategy):
        self._strategy = strategy

    def execute(self):
        result = self._strategy.run_strategy()
        return result


class Strategy(ABC):

    def __init__(self, ctx):
        self.ctx = ctx.obj
        self.org_name = self.ctx['org_name']
        self.org_pat = self.ctx['org_pat']

    @abstractmethod
    def run_strategy(self):
        pass


from setuptools import setup, find_packages

with open("README.md", "r") as f:
    long_description = f.read()

setup(
    name="azdo-cli",
    version="1.0.1",
    author="Migz Estrella",
    author_email="me@migzestrella.com",
    description="An Azure DevOps CLI client.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/migzpogi/azdo-cli",
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
    include_package_data=True,
    install_requires=[
        'Click',
    ],
    entry_points='''
    [console_scripts]
    azdocli=azdocli.azdocli:cli
    ''',
)


from azdocli.lib.commons import create_azdo_connection
from azdocli.lib.exceptions import exception_handler


class PipelinesAPI:
    @exception_handler
    def __init__(self, org_name, org_pat):
        self.connection = create_azdo_connection(org_name, org_pat)
        self.core_client = self.connection.clients_v6_0.get_pipelines_client()

    @exception_handler
    def get_pipelineruns(self, project_id, pipeline_id):
        return (project_id, pipeline_id)


import unittest
import os.path

from click.testing import CliRunner

from azdocli.azdocli import set_context, cli


class DummyContext():

    def __init__(self):
        self.obj = {}


class TestAzdoCli(unittest.TestCase):

    def test_if_set_context_was_called_then_it_should_update_its_values(self):
        context = DummyContext()
        set_context(context, 'dummy_operation', filename='./tests/lib/test-settings.ini')

        expected_org_name = 'dummy_org_name'
        expected_org_pat = 'dummy_pat'
        expected_operation = 'dummy_operation'

        actual_org_name = context.obj['org_name']
        actual_org_pat = context.obj['org_pat']
        actual_operation = context.obj['operation']

        self.assertEqual(expected_org_name, actual_org_name)
        self.assertEqual(expected_org_pat, actual_org_pat)
        self.assertEqual(expected_operation, actual_operation)

    def test_if_set_context_was_called_and_no_settings_ini_file_was_found_then_it_should_return_none(self):
        context = DummyContext()
        expected = None
        actual = set_context(context, 'dummy_operation', filename='./tests/lib/dummy/test-settings.ini')

        self.assertEqual(expected, actual)

    def test_if_cli_was_called_then_it_should_exit_code_0(self):
        runner = CliRunner()
        result = runner.invoke(cli)
        self.assertEqual(result.exit_code, 0)

    def test_if_init_was_called_then_it_should_create_a_settings_ini_file(self):
        filename = './tests/lib/dummy/init_test_settings.ini'
        runner = CliRunner()
        result = runner.invoke(cli, ['init',
                                     '--orgname', 'org_name_init_test',
                                     '--pat', 'pat_init_test',
                                     '--filename', filename])

        self.assertEqual(result.exit_code, 0)
        self.assertTrue(os.path.isfile(filename))

    def test_if_projects_was_called_then_it_should_exit_code_0(self):
        runner = CliRunner()
        result = runner.invoke(cli, ['projects'])
        self.assertEqual(result.exit_code, 0)

    def test_if_svc_was_called_then_it_should_exit_code_0(self):
        runner = CliRunner()
        result = runner.invoke(cli, ['svc'])
        self.assertEqual(result.exit_code, 0)


if __name__ == '__main__':
    unittest.main()


from azure.devops.exceptions import AzureDevOpsServiceError, AzureDevOpsAuthenticationError
import functools


def exception_handler(func):
    @functools.wraps(func)
    def inner_function(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except AzureDevOpsServiceError as e:
            print(e)
            exit(1)
        except AzureDevOpsAuthenticationError as e:
            print(e)
            exit(2)

    return inner_function
